from django.db.models.signals import post_save
from django.contrib.auth.models import User
from django.dispatch import receiver
from .models import Profile
from .models import WeightRecord
from .models import LiftRecord
import math

@receiver(post_save, sender=User)
def create_profile(sender, instance, created, **kwargs):
    if created:
        Profile.objects.create(user=instance)


@receiver(post_save, sender=WeightRecord)
def save_weight(sender, instance, **kwargs):
    Profile.objects.filter(user=instance.user).update(weight=instance.lbs)

@receiver(post_save, sender=LiftRecord)
def calculate_one_rep_max(sender, instance, **kwargs):
    if instance.weight or instance.reps:
        one_rm = int(instance.weight) * (1+0.025*instance.reps)
        LiftRecord.objects.filter(user=instance.user, name=instance.name, date=instance.date).update(one_rep_max_equiv=one_rm)


@receiver(post_save, sender=Profile)
def recalculate_goals(sender, instance, **kwargs):
    current_weight = int(instance.weight)
    current_activity_level = float(instance.activity_level)
    current_weight_change = float(instance.goal_weight_change)

    new_daily_cals = current_weight*current_activity_level*10+current_weight_change*500
    new_pct_daily_prot = math.ceil((instance.weight *4/new_daily_cals)*100)
    new_pct_daily_carb = math.ceil((100-new_pct_daily_prot)*2/3)
    new_pct_daily_fat = math.floor((100-new_pct_daily_prot)/3)
    new_daily_prot = (new_pct_daily_prot/100)*new_daily_cals/4
    new_daily_carb = (new_pct_daily_carb/100)*new_daily_cals/4
    new_daily_fat = (new_pct_daily_fat/100)*new_daily_cals/9

    Profile.objects.filter(user=instance.user).update(daily_cal_in=new_daily_cals,
                                                      daily_carbs=new_daily_carb,
                                                      daily_fat=new_daily_fat,
                                                      daily_protein=new_daily_prot,
                                                      pct_diet_carbs=new_pct_daily_carb,
                                                      pct_diet_fat=new_pct_daily_fat,
                                                      pct_diet_prot=new_pct_daily_prot)



from django import forms
from django.contrib.auth.models import User
from django.contrib.auth.forms import UserCreationForm
from .models import Profile
from .models import WeightRecord
from .models import LiftRecord
from .models import Food
from django.utils import timezone

WORKOUT_CATEGORIES = [
    ('All', 'All'),
    ('Abs', 'Abs'),
    ('Biceps', 'Biceps'),
    ('Butt/Hip', 'Butt/Hip'),
    ('Calves', 'Calves'),
    ('Chest', 'Chest'),
    ('Lats', 'Lats'),
    ('Shoulders', 'Shoulders'),
    ('Trapezius', 'Trapezius'),
    ('Triceps', 'Triceps'),
    ('Quads', 'Quads'),
]

MEAL_CATEGORIES = [
    ('All', 'All'),
    ('dinner-recipes', 'Dinner Recipes'),
    ('dessert', 'Dessert'),
    ('side-dishes', 'Sides'),
    ('appetizers', 'Appetizers'),
    ('breakfast-brunch', 'Breakfast & Brunch'),
    ('snacks', 'Snacks'),
    ('lunch', 'Lunch'),
    ('soup', 'Soup'),
    ('salad', 'Salad'),
    ('drinks', 'Drinks'),
    ]


class UserRegisterForm(UserCreationForm):
    email = forms.EmailField()

    class Meta:
        model = User
        fields = ['username', 'email', 'password1', 'password2']


class ProfileForm(forms.ModelForm):
    class Meta:
        model = Profile
        #we probably want to change the labels of the fields on the front end
        #example: daily_cal_in = forms.IntegerField(label="Daily Calories")
        fields = ['daily_cal_in', 'daily_carbs', 'daily_fat', 'daily_protein','weight', 'goal_weight_change','activity_level']


class WeightForm(forms.ModelForm):
    class Meta:
        model = WeightRecord
        fields = ['lbs', 'date']


class LiftForm(forms.ModelForm):
    class Meta:
        model = LiftRecord
        fields = ['name','weight', 'sets', 'reps', 'date']


class ExerciseFilterForm(forms.Form):
    category = forms.CharField(label='Filter by Category: ', widget=forms.Select(choices = WORKOUT_CATEGORIES))
    

class FoodForm(forms.ModelForm):
    class Meta:
        model = Food
        fields = ['name','carbs', 'fats', 'protein', 'date']


class SingleFood(forms.Form):
    foodName = forms.CharField(label='Search for a specific food')
    date = forms.DateField(label='Date', initial=timezone.now)


class OptionForm(forms.Form):
    optionName = forms.ChoiceField()


class MealFilterForm(forms.Form):
    category = forms.CharField(label='Filter by Meal Type: ', widget=forms.Select(choices = MEAL_CATEGORIES))


class ProfileUpdateForm(forms.ModelForm):
    class Meta:
        model = Profile
        fields = ['activity_level', 'goal_weight_change']



from setuptools import setup, find_packages,find_namespace_packages
import GymBuddyApp

setup(

    name = 'GymBuddy',
    version = '2.1.7',
    packages= find_namespace_packages(),
    url = 'https://github.com/CTKogstrom/GymBuddyApp',
    install_requires= ['asgiref==3.2.10','beautifulsoup4==4.9.3', 'html5lib==1.1','idna==2.10','bs4==0.0.1','certifi==2020.6.20','cycler==0.10.0',
                       'Django==3.1.2','django-crispy-forms==1.9.2','dj-database-url==0.5.0','kiwisolver==1.2.0','matplotlib==3.3.2','numpy==1.19.2',
                       'Pillow==7.2.0','pyparsing==2.4.7','python-dateutil==2.8.1','pytz==2020.1','requests==2.24.0','six==1.15.0','soupsieve==2.0.1',
                       'sqlparse==0.3.1','urllib3==1.25.11','webencodings==0.5.1','whitenoise==5.2.0','cchardet==2.1.7', 'lxml==4.6.1'],

    package_data= {'': ['*.html','*.json','*.css','*.jpg','*.png','*.zip','*.txt','README.md', '*.sqlite3'],
                   },
    entry_points={
        "console_scripts":['launch= GymBuddyApp.manage:main']
    },
    python_requires='>=3.6',
)